from django.contrib.syndication.views import Feed
from django.core.urlresolvers import reverse
from django.core.exceptions import ObjectDoesNotExist

from blogango.models import Blog, BlogEntry
from taggit.models import Tag

class main_feed(Feed):
    try:
        blog = Blog.objects.all()[0]
    except:
        class DummyBlog:
            def __init__ (self):         
                self.title = ''
                self.tag_line = ''
        blog = DummyBlog()
        
    title = blog.title
    link = "/rss/latest/"
    description = blog.tag_line
    
    def items (self):
        return BlogEntry.objects.filter(is_published=True, is_page=False)[:10]

    def item_description(self, item):
        return item.text.raw
    
class CatFeed(Feed):
    def get_object(self, bits):
        if len(bits) != 1:
            raise ObjectDoesNotExist
        return Tag.objects.get(name__iexact=bits[0])
    
    def title(self, obj):
        return "%s" % obj.name
    
    def link(self, obj):
        if not obj:
            raise ObjectDoesNotExist
        return reverse('blogango_tag_details', args=[obj.slug])
    
    def description(self, obj):
        return "Category: %s" % obj.name
    
    def items(self, obj):
        return BlogEntry.objects.filter(is_published=True, tags__in=[obj], is_page=False)



import datetime

from haystack import indexes
from blogango.models import BlogEntry


class BlogEntryIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.CharField(model_attr='text', document=True, use_template=True)
    title = indexes.CharField(model_attr='title')
    summary = indexes.CharField(model_attr='summary')
    created_by = indexes.CharField(model_attr='created_by') 
    
    meta_keywords = indexes.CharField(model_attr='meta_keywords') 
    meta_description = indexes.CharField(model_attr='meta_description')
    
    tags = indexes.CharField(model_attr='tags')

    def get_model(self):
        return BlogEntry

    def index_queryset(self):
        """Used when the entire index for model is updated."""
        return self.get_model().objects.filter(publish_date__lte=datetime.datetime.now())



import re

from django import template
from django.conf import settings
from django.core.urlresolvers import reverse
from django.contrib.sites.models import Site

from taggit.models import Tag, TaggedItem
from blogango.views import _get_archive_months
from blogango.models import Blog

register = template.Library()

class BlogangoContext(template.Node):
    def __init__(self):
        pass

    def render(self, context):
        #only one blog must be present
        blog = Blog.objects.get_blog()
        tags = Tag.objects.all()
        tags = [tag for tag in tags if tag.taggit_taggeditem_items.all().count()>2]
        feed_url = reverse('blogango_feed')
        archive_months = _get_archive_months()
        site = Site.objects.get_current()

        extra_context = {
                         'tags': tags,
                         'feed_url': feed_url,
                         'archive_months': archive_months,
                         'blog': blog,
                         'site': site,
                         }
        context.update(extra_context)
        return ''

def blogango_extra_context(parser, token):
    return BlogangoContext()


@register.assignment_tag
def related_posts(post):
    tags = post.tags.all()
    related_posts = TaggedItem.objects.filter(tag__in=tags).filter(
        blogentry__is_published=True).exclude(blogentry=post
        ).distinct().order_by("-blogentry__created_on")[:5]
    return set([el.content_object for el in related_posts])

#django snippets #2107
@register.filter(name='twitterize')
def twitterize(token):
    return re.sub(r'\W(@(\w+))', r'<a href="https://twitter.com/\2">\1</a>', token)
twitterize.is_safe = True

register.tag('blogango_extra_context', blogango_extra_context)

@register.filter
def truncate_chars(ip_string, length=30):
    length, dots = int(length), 3
    if len(ip_string) < length:return ip_string
    else:return ip_string[:length-dots] + '.' * dots




from datetime import date, datetime, time
from time import strptime

from django.core.urlresolvers import reverse
from django.db.models import signals

from pingback import register_pingback, ping_func
from pingback.client import ping_external_links, ping_directories
from django_xmlrpc import xmlrpcdispatcher

from blogango.models import BlogEntry

def pingback_blog_handler(year, month, slug, **kwargs):
    return BlogEntry.objects.get(created_on__year=year, 
                                 created_on__month=month, 
                                 slug=slug, 
                                 is_published=True)
    

# ping_details = {'blogango_details': pingback_blog_handler}

register_pingback('blogango.views.details', pingback_blog_handler)

xmlrpcdispatcher.register_function(ping_func, 'pingback.ping')

# ping external links in the entry

def get_blog_text(instance):
    return instance.text.rendered

signals.post_save.connect(ping_external_links(content_func=get_blog_text, url_attr='get_absolute_url'), sender=BlogEntry, weak=True)


from django.contrib.auth.models import User
from django import forms
from django.contrib.admin import widgets

from blogango.models import Blog, BlogRoll, BlogEntry

class WideTextArea(forms.Textarea):
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('attrs',{}).update({'rows': '20', 'cols':'80'})
        super(WideTextArea, self).__init__(*args, **kwargs)

class EntryForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(EntryForm, self).__init__(*args, **kwargs)
        self.fields['created_by'].queryset = User.objects.filter(is_staff=True)
        self.fields['publish_date'].widget = widgets.AdminSplitDateTime()

    title = forms.CharField(max_length=100, required=False, widget=forms.TextInput(attrs={'size':'40'}))
    text = forms.CharField(widget=WideTextArea(attrs={'class': 'resizable'}))

    class Meta:
        model = BlogEntry
        exclude = ('slug', 'summary')

class CommentForm(forms.Form):
    text = forms.CharField(widget=forms.Textarea(attrs={'class': 'textarea'}), label='Comment')
    name = forms.CharField(max_length=100, widget=forms.TextInput(attrs={'class': 'textfield'}))
    url = forms.URLField(required=False, widget=forms.TextInput(attrs={'class': 'textfield'}))
    email = forms.EmailField(widget=forms.TextInput(attrs={'class': 'textfield'}))

class InstallForm(forms.ModelForm):

    class Meta:
        model = Blog
        exclude = ('entries_per_page', 'recents', 'recent_comments')

class PreferencesForm(forms.ModelForm):

    class Meta:
        model = Blog

class BlogForm(forms.ModelForm):

    class Meta:
        model = BlogRoll
        exclude = ('is_published',)




from django.core.management.base import NoArgsCommand
from django.http import HttpRequest
from django.conf import settings
from django.contrib.sites.models import Site

from blogango.models import BlogEntry, Reaction

import urllib2

try:
    import simplejson
except ImportError:
    import django.utils.simplejson

BACKTYPE_URL = "http://api.backtype.com/comments/connect.json?url=%s&key=%s"

class Command(NoArgsCommand):
    help = """
            Sync reactions from backtype
            
            Supports all sources from backtype
            Sample cron job call:
                python manage.py update_reactions
           """

    def handle_noargs(self, **options):
        current_site = Site.objects.get_current() # hack to get absolute uri for the blog entries
        for entry in BlogEntry.objects.all():
            entry_url = current_site.domain + entry.get_absolute_url()
            print "getting backtype results for %s" % (entry_url)
            url = BACKTYPE_URL % (entry_url, settings.BACKTYPE_API_KEY)
            resp = urllib2.urlopen(url)
            json_data = simplejson.load(resp)            

            comments = json_data["comments"]
            for comment in comments:
                if comment['entry_type'] == 'tweet':
                    pk = comment['tweet_id']
                    created_on = comment['tweet_created_at']
                    text = comment['tweet_text']
                    user_name = comment['tweet_from_user']
                    profile_image = comment['tweet_profile_image_url']
                    user_url = "http://twitter.com/%s" %(user_name)
                    source = 'twitter'
                else:
                    pk = comment['comment']['id']
                    created_on = comment['comment']['date']
                    text = comment['comment']['content']
                    user_name = comment['author']['name']
                    user_url = comment['author']['url']
                    source = comment['entry_src']
                    if source == 'yc':
                        profile_image = "http://mediacdn.disqus.com/images/reactions/services/hackernews_128.png"
                    else:
                        profile_image = "http://mediacdn.disqus.com/images/reactions/services/%s_128.png" %(source)
                try:
                    reaction = Reaction.objects.get(pk=pk)
                except Reaction.DoesNotExist:
                    reaction = Reaction()
                    reaction.reaction_id = pk
                    reaction.comment_for = entry
                    reaction.created_on = created_on
                    reaction.text = text
                    reaction.user_name = user_name
                    reaction.user_url = user_url
                    reaction.profile_image = profile_image
                    reaction.source = source
                    reaction.save()




from django.shortcuts import render_to_response, get_object_or_404, redirect
from django.http import HttpResponseRedirect, HttpResponse
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.template import RequestContext
from django.core.paginator import Paginator
from django.core.urlresolvers import reverse
from django.views.generic.date_based import archive_month
from django.http import Http404
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.admin.views.decorators import staff_member_required
from django.utils.encoding import smart_str
from django.db.models import Q
from datetime import datetime
from django.views.decorators.http import require_POST
from django.utils import simplejson as json

from taggit.models import Tag

from blogango.models import Blog, BlogEntry, Comment, BlogRoll, Reaction
from blogango import forms as bforms
from blogango.conf.settings import AKISMET_COMMENT, AKISMET_API_KEY
from blogango.akismet import Akismet, AkismetError

@staff_member_required
def admin_dashboard(request):
    recent_drafts = BlogEntry.default.filter(is_published=False).order_by('-created_on')[:5]
    recent_entries = BlogEntry.objects.order_by('-created_on')[:5]
    return render('blogango/admin/index.html', request, {'recent_drafts': recent_drafts,
                                                         'recent_entries': recent_entries})

@staff_member_required
def admin_entry_edit(request, entry_id=None):
    entry = None
    entry_form = bforms.EntryForm(initial={'created_by': request.user.id, 'publish_date':datetime.now()})
    if entry_id:
        entry = get_object_or_404(BlogEntry, pk=entry_id)
        entry_form = bforms.EntryForm(instance=entry, initial={'text': entry.text.raw})
    if request.POST:
        entry_form = bforms.EntryForm(request.POST, instance=entry)

        if entry_form.is_valid():
            new_entry = entry_form.save(commit=False)
            if "publish" in request.POST:
                new_entry.is_published = True
            if "page" in request.POST:
                new_entry.is_page = True
            new_entry.save()
            tag_list = entry_form.cleaned_data['tags']
            new_entry.tags.set(*tag_list)
            if new_entry.is_published:
                return redirect(new_entry)
            return redirect(reverse('blogango_admin_entry_edit', args=[new_entry.id])+'?done')
    tags_json = json.dumps([each.name for each in Tag.objects.all()])
    return render('blogango/admin/edit_entry.html', request, {'entry_form': entry_form,
                                                              'entry': entry,
                                                              'tags_json': tags_json})

@staff_member_required
def admin_manage_entries(request, username=None):
    author = None
    if username:
        author = get_object_or_404(User, username=username)
        entries = BlogEntry.default.filter(created_by=author) 
    else:
        entries = BlogEntry.default.all()
    return render('blogango/admin/manage_entries.html', request, {'entries': entries, 'author': author})

@staff_member_required
def admin_manage_comments(request, entry_id=None):
    # fetch all comments, objects gets you only public ones
    blog_entry = None
    if entry_id:
        blog_entry = get_object_or_404(BlogEntry, pk=entry_id)
    if 'blocked' in request.GET:
        comments = Comment.default.filter(Q(is_spam=True)|Q(is_public=False)).order_by('-created_on')
    else:
        comments = Comment.objects.order_by('-created_on')
    if blog_entry:
        comments = comments.filter(comment_for=blog_entry)
    page = request.GET.get('page', 1)
    comments_per_page = getattr(settings, 'COMMENTS_PER_PAGE', 20)
    paginator = Paginator(comments, comments_per_page)
    page_ = paginator.page(page)
    comments = page_.object_list
    return render('blogango/admin/manage_comments.html', request, {'comments': comments, 'blog_entry': blog_entry, 'page_': page_})

@staff_member_required
def admin_edit_preferences(request):
    #only one blog must be present
    blog = Blog.objects.get_blog()
    form = bforms.PreferencesForm(instance=blog)
    if request.POST:
        form = bforms.PreferencesForm(request.POST, instance=blog)
        if form.is_valid():
            form.save()
            return redirect(request.path+"?done")
    return render('blogango/admin/edit_preferences.html', request, {'form': form})

@staff_member_required
@require_POST
def admin_comment_approve(request):
    comment_id = request.POST.get('comment_id', None)
    comment = get_object_or_404(Comment, pk=comment_id)
    comment.is_spam = False
    comment.is_public = True
    comment.save()
    return HttpResponse(comment.pk)

@staff_member_required
@require_POST
def admin_comment_block(request):
    comment_id = request.POST.get('comment_id', None)
    comment = get_object_or_404(Comment, pk=comment_id)
    comment.is_public = False
    comment.save()
    return HttpResponse(comment.pk)



def handle404(view_function):
    def wrapper(*args, **kwargs):
        try:
            return view_function(*args, **kwargs)
        except ObjectDoesNotExist:
            raise Http404
    return wrapper


def index(request, page = 1):
    if not _is_blog_installed():
        return HttpResponseRedirect(reverse('blogango_install'))
    page = int(page)
    blog = Blog.objects.all()[0]
    entries = BlogEntry.objects.filter(is_page=False).order_by('-created_on')
    paginator = Paginator(entries, blog.entries_per_page)
    if paginator.num_pages < page:
        return redirect(reverse('blogango_page', args=[paginator.num_pages]))
    page_ = paginator.page(page)
    entries = page_.object_list
    payload = {'entries': entries, 'page_': page_}
    return render('blogango/mainpage.html', request, payload)


def check_comment_spam(request, comment):
    api = Akismet(AKISMET_API_KEY, 'http://%s' % (request.get_host()), request.META.get('HTTP_USER_AGENT', ''))

    message = "Akismet API key is invalid."
    try:
        is_verified = api.verify_key()
    except AkismetError, e:
        is_verified = False
    if is_verified:
        akismet_data = {'user_ip': request.META['REMOTE_ADDR'],
                        'user_agent': request.META.get('HTTP_USER_AGENT', ''),
                        'comment_author': smart_str(comment.user_name),
                        'comment_author_email': smart_str(comment.email_id),
                        'comment_author_url': smart_str(comment.user_url),
                        'comment_type': 'comment'}

        return api.comment_check(smart_str(comment.text), akismet_data)
    raise AkismetError(message)



@handle404
def details(request, year, month, slug):
    if not _is_blog_installed():
        return HttpResponseRedirect(reverse('blogango_install'))

    entry = BlogEntry.default.get(created_on__year=year,
                                  created_on__month=month,
                                  slug=slug)

    # published check needs to be handled here to allow previews
    if not entry.is_published:
        if request.user.is_staff and 'preview' in request.GET:
            pass
        else:
            raise Http404

    if request.method == 'POST':
        comment_f = bforms.CommentForm(request.POST)
        if comment_f.is_valid():
            comment_by = request.user if request.user.is_authenticated() else None
            comment = Comment(text=comment_f.cleaned_data['text'],
                              created_by=comment_by,
                              comment_for=entry,
                              user_name=comment_f.cleaned_data['name'],
                              user_url=comment_f.cleaned_data['url'],
                              email_id=comment_f.cleaned_data['email'])
            comment.is_public = getattr(settings, 'AUTO_APPROVE_COMMENTS', True)
            if AKISMET_COMMENT:
                try:
                    comment.is_spam = check_comment_spam(request, comment)
                except AkismetError:
                    # Most likely could be a timeout to a spam message
                    comment.is_spam = True
            if not comment.is_spam:
                request.session["name"] = comment_f.cleaned_data['name']
                request.session["email"] = comment_f.cleaned_data['email']
                request.session["url"] = comment_f.cleaned_data['url']
            comment.save()
            return HttpResponseRedirect('#comment-%s' % comment.pk)
    else:
        init_data = {}
        if request.user.is_authenticated():
            init_data['name'] = request.user.get_full_name() or request.user.username
            init_data['email'] = request.user.email
        else:
            init_data['name'] = request.session.get("name", "")
            init_data['email'] = request.session.get("email", "")
            init_data['url'] = request.session.get("url", "")
        comment_f = bforms.CommentForm(initial=init_data)

    comments = Comment.objects.filter(comment_for=entry, is_spam=False)
    reactions = Reaction.objects.filter(comment_for=entry)
    # tags = Tag.objects.filter(tag_for=entry)
    payload = {'entry': entry, 'comments': comments, 'reactions': reactions, 'comment_form': comment_f}
    return render('blogango/details.html', request, payload)


@handle404
def page_details(request, slug):
    if not _is_blog_installed():
        return HttpResponseRedirect(reverse('blogango_install'))

    entry = BlogEntry.default.get(is_page=True,
                                  slug=slug)

    # published check needs to be handled here to allow previews
    if not entry.is_published:
        if request.user.is_staff and 'preview' in request.GET:
            pass
        else:
            raise Http404

    if request.method == 'POST':
        comment_f = bforms.CommentForm(request.POST)
        if comment_f.is_valid():
            comment_by = request.user if request.user.is_authenticated() else None
            comment = Comment(text=comment_f.cleaned_data['text'],
                              created_by=comment_by,
                              comment_for=entry,
                              user_name=comment_f.cleaned_data['name'],
                              user_url=comment_f.cleaned_data['url'],
                              email_id=comment_f.cleaned_data['email'])
            comment.is_public = getattr(settings, 'AUTO_APPROVE_COMMENTS', True)
            if AKISMET_COMMENT:
                try:
                    comment.is_spam = check_comment_spam(request, comment)
                except AkismetError:
                    # Most likely spam causing timeout error.
                    comment.is_spam = True
            comment.save()
            return HttpResponseRedirect('#comment-%s' % comment.pk)
    else:
        init_data = {'name': None}
        if request.user.is_authenticated():
            init_data['name'] = request.user.get_full_name() or request.user.username
        comment_f = bforms.CommentForm(initial=init_data)

    comments = Comment.objects.filter(comment_for=entry, is_spam=False)
    reactions = Reaction.objects.filter(comment_for=entry)
    # tags = Tag.objects.filter(tag_for=entry)
    payload = {'entry': entry, 'comments': comments, 'reactions': reactions, 'comment_form': comment_f}
    return render('blogango/details.html', request, payload)


def tag_details(request, tag_slug, page=1):
    tag = get_object_or_404(Tag, slug=tag_slug)
    page = int(page)
    blog = Blog.objects.get_blog()
    tagged_entries = BlogEntry.objects.filter(is_published=True, tags__in=[tag])
    paginator = Paginator(tagged_entries, blog.entries_per_page)
    if page>paginator.num_pages:
        return redirect(reverse('blogango_tag_details_page', args=[tag.slug, paginator.num_pages]))
    page_ = paginator.page(page)
    entries = page_.object_list
    payload = {'tag': tag, 'entries': entries}
    payload['page_'] = page_
    return render('blogango/tag_details.html', request, payload)

@login_required
def install_blog(request):
    if _is_blog_installed():
        return HttpResponseRedirect(reverse('blogango_index'))

    if request.method == 'GET':
        install_form = bforms.InstallForm()
    if request.method == 'POST':
        install_form = bforms.InstallForm(request.POST)
        if install_form.is_valid():
            install_form.save()
            return HttpResponseRedirect(reverse('blogango_index'))
    payload = {"install_form": install_form}
    return render('blogango/install.html', request, payload)


@login_required
def  create_blogroll(request):
    if request.method == 'GET':
        blogroll_form = bforms.BlogForm()
    if request.method == 'POST':
        blogroll_form = bforms.BlogForm(request.POST)
        if blogroll_form.is_valid():
            blogroll_form.save()
            return HttpResponseRedirect('.')
    payload = {"blogroll_form": blogroll_form}
    return render('blogango/blogroll.html', request, payload)


def author(request, username, page=1):
    page = int(page)
    blog = Blog.objects.get_blog()
    author = get_object_or_404(User, username=username)
    author_posts = author.blogentry_set.filter(is_published=True)
    paginator = Paginator(author_posts, blog.entries_per_page)
    if page>paginator.num_pages:
        return redirect(reverse('blogango_author_page', args=[author.username, paginator.num_pages]))
    page_ = paginator.page(page)
    entries = page_.object_list
    return render('blogango/author.html', request, {'author': author,
                                                    'entries': entries,
                                                    'page_': page_})

def monthly_view(request, year, month):
    queryset = BlogEntry.objects.filter(is_page=False, is_published=True)
    return archive_month(request=request,
                         template_name='blogango/archive_view.html',
                         year=year,
                         month=month,
                         queryset=queryset,
                         date_field='created_on',
                         allow_empty=True,
                         extra_context=_get_sidebar_objects(request))


#Helper methods.
def _is_blog_installed():
    if Blog.objects.count() == 0:
        return False
    return True


def render (template, request, payload):
    """Wrapper on render_to_response.
    Adds sidebar objects. Adds RequestContext"""
    payload.update(_get_sidebar_objects(request))
    return render_to_response(template, payload, context_instance=RequestContext(request),)


def _get_sidebar_objects (request):
    """Gets the objects which are always displayed in the sidebar"""
    try:
        blog = Blog.objects.all()[0]
    except:
        return {}
    recents = BlogEntry.objects.filter(is_page = False, is_published = True).order_by('-created_on')[:blog.recents]
    blogroll = BlogRoll.objects.filter(is_published=True)
    return {'blog':blog,
            'recents':recents,
            'blogroll':blogroll,
            'canonical_url': request.build_absolute_uri(),
            'pingback_xmlrpc_url': request.build_absolute_uri(reverse('xmlrpc')),}


def _get_archive_months():
    """Get the months for which at least one entry exists"""
    dates = BlogEntry.objects.filter(is_page=False, is_published=True).dates('created_on', 'month', order='DESC')
    return dates


def _generic_form_display(request, form_class):
    if request.method == 'GET':
        form_inst = form_class()
    if request.method == 'POST':
        form_inst = form_class(request.POST)
        if form_inst.is_valid():
            form_inst.save()
            return HttpResponseRedirect('.')
    payload = {"install_form": form_inst}
    return render('blogango/install.html', request, payload)


def generic(request): # A generic form processor.
    if request.method == 'GET':
        pass
    if request.method == 'POST':
        pass




from datetime import datetime
from django.conf.urls.defaults import *
from django.contrib.sitemaps import GenericSitemap
from blogango import feeds
from blogango.models import BlogEntry

blog_info_dict = {
    'queryset': BlogEntry.objects.filter(is_published=True, publish_date__lte=datetime.now),
    'date_field': 'publish_date',
}
sitemaps = {
    'blog': GenericSitemap(blog_info_dict, priority=0.5)
}

urlpatterns = patterns('blogango.views',
    url(r'^$', 'index', name='blogango_index'),
    url(r'^install/$', 'install_blog', name='blogango_install'),
    url(r'^page/(?P<page>\d+)/$', 'index',  name='blogango_page'),
    url(r'^(?P<year>\d{4})/(?P<month>\d{2})/(?P<slug>[-\w]+)/$', 'details', name='blogango_details'),
    url(r'^blogroll/$', 'create_blogroll', name='blogango_blogroll'),
    url(r'^tag/(?P<tag_slug>[-\w]+)/$','tag_details', name='blogango_tag_details'),
    url(r'^tag/(?P<tag_slug>[-\w]+)/(?P<page>\d+)/$','tag_details', name='blogango_tag_details_page'),
    url(r'^author/(?P<username>[\w.@+-]+)/$', 'author', name='blogango_author'),
    url(r'^author/(?P<username>[\w.@+-]+)/(?P<page>\d+)/$', 'author', name='blogango_author_page'),

    url(r'^admin/$', 'admin_dashboard', name='blogango_admin_dashboard'),
    url(r'^admin/entry/new/$', 'admin_entry_edit', name='blogango_admin_entry_new'),
    url(r'^admin/entry/edit/(?P<entry_id>\d+)/$', 'admin_entry_edit', name='blogango_admin_entry_edit'),
    url(r'^admin/entry/manage/$', 'admin_manage_entries', name='blogango_admin_entry_manage'),
    url(r'^admin/entry/manage/(?P<username>[\w.@+-]+)/$', 'admin_manage_entries', name='blogango_admin_author_entry_manage'),
    url(r'^admin/comments/manage/$', 'admin_manage_comments', name='blogango_admin_comments_manage'),
    url(r'^admin/comments/manage/(?P<entry_id>\d+)/$', 'admin_manage_comments', name='blogango_admin_entry_comments_manage'),
    url(r'^admin/preferences/edit/$', 'admin_edit_preferences', name='blogango_admin_edit_preferences'),
    url(r'^admin/comment/approve/$', 'admin_comment_approve', name='blogango_admin_comment_approve'),
    url(r'^admin/comment/block/$', 'admin_comment_block', name='blogango_admin_comment_block'),
)

#search view
urlpatterns += patterns('',
    #url(r'^search/', include('haystack.urls')),
)

# sitemap.xml
urlpatterns += patterns('django.contrib.sitemaps.views',
    url(r'^sitemap\.xml$', 'sitemap', {'sitemaps': sitemaps}),
)

# Archive view
urlpatterns += patterns('blogango.views',
    url(r'^archive/(?P<year>\d+)/(?P<month>\w+)/$', 'monthly_view', name='blogango_archives')
)

urlpatterns += patterns('django_xmlrpc.views',
    url(r'^xmlrpc/$', 'handle_xmlrpc', {}, name='xmlrpc'),
)

urlpatterns += patterns('',
    url(r'^rss/latest/$', feeds.main_feed(), name='blogango_feed')
)

urlpatterns += patterns('blogango.views',
    url(r'^(?P<slug>[-\w]+)/$', 'page_details', name='blogango_page_details'),
)



from django.test  import TestCase
from django.test.client import Client
from datetime import datetime
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.contrib.staticfiles import finders
from django.contrib.staticfiles.storage import staticfiles_storage

from taggit.models import Tag
from models import Blog, BlogEntry, Comment


class BlogTestCase(TestCase):

    def setUp(self):
        self.c = Client()

    def test_bloginstall_redirect(self):
        "check that the blog redirects to install page when there is no blog installed"
        response = self.c.get(reverse("blogango_index"))
        self.assertEqual(response.status_code, 302)

    def test_blogindex_access(self):
        Blog.objects.create(title='test', tag_line="blog",
                entries_per_page=10, recents=5, recent_comments=5)
        response = self.c.get(reverse("blogango_index"))
        self.assertEqual(response.status_code, 200)
        User.objects.create_user(username='foo', password='bar')
        self.c.login(username='foo', password='bar')
        response = self.c.get(reverse('blogango_install'))
        self.assertEqual(response.status_code, 302)

    def test_bloginstall(self):
        User.objects.create_user(username='foo', password='bar')
        response = self.c.get(reverse('blogango_install'))
        self.assertEqual(response.status_code, 302)
        self.c.login(username='foo', password='bar')
        response = self.c.get(reverse('blogango_install'))
        self.assertEqual(response.status_code, 200)

    def test_single_existence(self):
        """Test that the blog is created only once """
        self.blog = Blog(title="test", tag_line="new blog", entries_per_page=10,
                        recents=5, recent_comments=5)
        self.blog.save()
        blog = Blog(title="test", tag_line="new blog", entries_per_page=10,
                    recents=5, recent_comments=5)
        #should raise Exception when another blog is created
        self.assertRaises(Exception, blog.save)
        blog = Blog.objects.get()
        blog.title = 'edited'
        blog.save()
        self.assertEqual(Blog.objects.count(), 1)
        self.assertEqual(Blog.objects.get().title, 'edited')


class TestBlogEntry(TestCase):
    def setUp(self):
        self.blog = Blog(title="test", tag_line="new blog", entries_per_page=10,
                    recents=5, recent_comments=5)
        self.blog.save()
        self.c = Client()
        self.user = User.objects.create_user(username='gonecrazy', email='gonecrazy@gmail.com', password='gonecrazy')
        self.user.is_staff = True
        self.user.save()

    def test_blogentry_managers(self):
        """Test custom manager works as expected"""
        e1 = BlogEntry.objects.create(title="test", text='foo', created_by=self.user,
                    publish_date=datetime.today(), text_markup_type='plain',
                    is_published=False)
        e2 = BlogEntry.objects.create(title="test", text='foo', created_by=self.user,
                    publish_date=datetime.today(), text_markup_type='plain',
                    is_published=True)
        self.assertEqual(BlogEntry.default.count(), 2)
        self.assertEqual(BlogEntry.objects.count(), 1)


class TestViews(TestCase):
    """Test Views  of the blog"""

    def setUp(self):
        self.blog = Blog(title="test", tag_line="new blog", entries_per_page=10,
                    recents=5, recent_comments=5)
        self.blog.save()
        self.c = Client()
        self.user = User.objects.create_user(username='gonecrazy', email='gonecrazy@gmail.com', password='gonecrazy')
        self.user.is_staff = True
        self.user.save()

    def test_first_page(self):
        response = self.c.get(reverse("blogango_index"))
        #it should return 200 for all users
        self.assertEqual(response.status_code, 200)

    def test_entries_pagination(self):
        num_entries = 15
        for each in range(num_entries):
            BlogEntry.objects.create(title="test", text='foo', created_by=self.user,
                    publish_date=datetime.today(), text_markup_type='plain')
        self.assertEqual(BlogEntry.objects.count(), 15)
        response = self.c.get(reverse('blogango_page', args=[1]))
        self.assertEqual(response.status_code, 200)
        response = self.c.get(reverse('blogango_page', args=[2]))
        self.assertEqual(response.status_code, 200)
        response = self.c.get(reverse('blogango_page', args=[3]))
        self.assertEqual(response.status_code, 302)

    def test_entries_slug(self):
        e1 = BlogEntry.objects.create(title="test", text='foo', created_by=self.user,
                    publish_date=datetime.today(), text_markup_type='plain')
        e2 = BlogEntry.objects.create(title="test", text='foo', created_by=self.user,
                    publish_date=datetime.today(), text_markup_type='plain')
        self.assertEqual(e1.slug, "test")
        self.assertEqual(e2.slug, "test-2")
        e1 = BlogEntry.objects.get(pk=e1.pk)
        e2 = BlogEntry.objects.get(pk=e2.pk)
        e1.save()
        self.assertEqual(e1.slug, "test")
        self.assertEqual(e2.slug, "test-2")
        e3 = BlogEntry.objects.create(title="test", text='foo', created_by=self.user,
                    publish_date=datetime.today(), text_markup_type='plain')
        self.assertEqual(e3.slug, "test-3")

    def test_tags_url(self):
        e1 = BlogEntry.objects.create(title="test", text='foo', created_by=self.user,
                    publish_date=datetime.today(), text_markup_type='plain')
        e2 = BlogEntry.objects.create(title="test", text='foo', created_by=self.user,
                    publish_date=datetime.today(), text_markup_type='plain')
        e1.tags.add("test1", "test2")
        e2.tags.add("test1")
        response = self.c.get(reverse('blogango_tag_details', args=['test1']))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['entries']), 2)
        response = self.c.get(reverse('blogango_tag_details', args=['test2']))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['entries']), 1)

    def test_entry_existence(self):
        """Test for the existence of entry"""
        BlogEntry.objects.create(**{'title': 'test post',
            'text': 'this is the test post',
            'publish_date': datetime.strptime("2011-09-22", "%Y-%m-%d"),
            'text_markup_type': "html",
            'created_by': self.user})
        response = self.c.get(reverse('blogango_details', args=['2011', '09', 'test-post']))
        self.assertEqual(response.status_code, 200)

    def test_add_comment(self):
        #test if a comment can be added to entry
        create_test_blog_entry(self.user)
        entry = BlogEntry.default.all()[0]
        reponse = self.c.post(entry.get_absolute_url(), {'name': 'gonecrazy', 'email': 'plaban@agiliq.com',
                                                          'text': 'this is a comment', 'button': 'Comment'})
        comment = Comment.objects.get(comment_for=entry)
        comments = Comment.objects.filter(comment_for=entry)
        self.assertEqual(comments.count(), 1)
        self.assertEqual(comment.text, 'this is a comment')

    def test_author_details(self):
        create_test_blog_entry(self.user)
        response = self.c.get("/blog/author/%s/" % self.user.username)
        entries = response.context['entries']
        self.assertEqual(1, entries.count())

    def test_author_entries_pagination(self):
        num_entries = 15
        for each in range(num_entries):
            BlogEntry.objects.create(title="test", text='foo', created_by=self.user,
                    publish_date=datetime.today(), text_markup_type='plain')
        self.assertEqual(BlogEntry.objects.count(), 15)
        response = self.c.get(reverse('blogango_author_page', args=[self.user.username, 1]))
        self.assertEqual(response.status_code, 200)
        response = self.c.get(reverse('blogango_author_page', args=[self.user.username, 2]))
        self.assertEqual(response.status_code, 200)
        response = self.c.get(reverse('blogango_author_page', args=[self.user.username, 3]))
        self.assertEqual(response.status_code, 302)

    def test_tagged_entries_pagination(self):
        num_entries = 15
        for each in range(num_entries):
            entry = BlogEntry.objects.create(title="test", text='foo', created_by=self.user,
                        publish_date=datetime.today(), text_markup_type='plain')
            entry.tags.add("taggit")
        tag = Tag.objects.get(name='taggit')
        self.assertEqual(BlogEntry.objects.count(), 15)
        response = self.c.get(reverse('blogango_tag_details_page', args=[tag.slug, 1]))
        self.assertEqual(response.status_code, 200)
        response = self.c.get(reverse('blogango_tag_details_page', args=[tag.slug, 2]))
        self.assertEqual(response.status_code, 200)
        response = self.c.get(reverse('blogango_tag_details_page', args=[tag.slug, 3]))
        self.assertEqual(response.status_code, 302)

    def tearDown(self):
        self.blog.delete()
        self.user.delete()


class TestAdminActions(TestCase):
    """check for admin action on the blog"""
    def setUp(self):
        self.c = Client()
        self.blog = Blog(title="test", tag_line="new blog", entries_per_page=10, recents=5, recent_comments=5)
        self.blog.save()
        self.user = User.objects.create_user(username='gonecrazy', email='gonecrazy@gmail.com', password='gonecrazy')
        self.user.is_staff = True
        self.user.save()
        response = self.c.login(username='gonecrazy', password='gonecrazy')
        entry = create_test_blog_entry(self.user)
        create_test_comment(entry)

    def test_check_adminpage(self):
        """Check that the admin page is accessible to only staff members"""
        response = self.c.get(reverse('blogango_admin_dashboard'))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed("admin/login.html")
        self.c.login(username='gonecrazy', password='gonecrazy')
        response = self.c.get(reverse('blogango_admin_dashboard'))
        self.assertTemplateNotUsed("admin/login.html")

    def test_add_entry(self):
        """Test whether a entry can be added to the blog """
        self.assertEqual(BlogEntry.objects.count(), 1)
        response = self.c.login(username='gonecrazy', password='gonecrazy')
        #Test posting invalid form
        data = {'text_markup_type': 'html', 'created_by':self.user.pk}
        response = self.c.post(reverse("blogango_admin_entry_new"), data)
        self.assertEqual(response.status_code, 200)
        self.assertFormError(response, "entry_form", "text", "This field is required.")
        self.assertFormError(response, "entry_form", "tags", "This field is required.")
        #post a valid form but don't publish it
        data['text'] = 'test text'
        data['publish_date_0'] = '2011-09-22'
        data['publish_date_1'] = '17:17:55'
        data['tags'] = 'testing, testing2'
        data['save'] = 'Save'
        response = self.c.post(reverse("blogango_admin_entry_new"), data)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(BlogEntry.default.count(), 2)
        self.assertEqual(BlogEntry.objects.count(), 1)
        #post valid data and publish it
        del data['save']
        data['publish'] = 'Save and Publish'
        response = self.c.post(reverse("blogango_admin_entry_new"), data)
        self.assertEqual(BlogEntry.objects.count(), 2)
        entry = BlogEntry.objects.latest('pk')
        #check proper tags have been set
        self.assertEqual([each.name for each in entry.tags.all()], ["testing", "testing2"])

    def test_edit_entry(self):
        """Test for editing a entry in the blog"""
        self.c.login(username='gonecrazy', password='gonecrazy')
        #edit a post .. the title is changed
        post = BlogEntry.objects.create(**{'title': 'test post',
                    'text': 'this is the test post',
                    'publish_date': datetime.strptime("2011-09-22", "%Y-%m-%d"),
                    'text_markup_type': "html",
                    'created_by': self.user})
        post.tags.add("test", "another test")
        self.assertEqual([each.name for each in post.tags.all()], ["test", "another test"])
        response = self.c.post("/blog/admin/entry/edit/%s/" % post.pk,
                {'title': 'the new test post', 'text': 'this is the test post',
                'publish_date_0': '2011-09-22', 'publish_date_1': '17:17:55',
                'text_markup_type': "html", 'created_by': self.user.pk, 'publish': 'Save and Publish',
                'tags': 'testing'})
        self.assertEqual(response.status_code, 302)
        entry = BlogEntry.objects.get(pk=post.pk)
        self.assertEqual(entry.title, "the new test post")
        self.assertEqual([each.name for each in entry.tags.all()], ["testing"])

    def test_change_preferences(self):
        """check if the admin can change the preferences of blog """
        self.c.post(reverse('blogango_admin_edit_preferences'), {'title': 'new Blog', 'tag_line': 'my new blog',
                                                                           'entries_per_page': 10, 'recents': 5, 'recent_comments': 5})
        blog = Blog.objects.get_blog()
        self.assertEqual(blog.title, 'new Blog')

    def test_admin_entrymanage(self):
        """Check if all the entries are retrieved to manage"""
        response = self.c.get(reverse('blogango_admin_entry_manage'))
        entries = response.context['entries']
        self.assertEqual(1, entries.count())

    def test_admin_author_entry_manage(self):
        """Check if we can manage entries by a specific author"""
        user = User.objects.create_superuser("test", "test@agiliq.com", "test")
        create_test_blog_entry(user)
        response = self.c.get(reverse('blogango_admin_author_entry_manage', args=["test"]))
        entries = response.context['entries']
        self.assertEqual(entries.count(), 1)
        self.assertEqual(BlogEntry.default.count(), 2)

    def test_admin_commentmanage(self):
        """check if all the comments are retrieved to manage"""
        response = self.c.get(reverse('blogango_admin_comments_manage'))
        comments = response.context['comments']
        self.assertEqual(1, comments.count())

    def test_manage_entry_comments(self):
        """Check if there is a page to see comments for a particulat entry"""
        user = User.objects.create_superuser("test", "test@agiliq.com", "test")
        entry = create_test_blog_entry(user)
        create_test_comment(entry)
        response = self.c.get(reverse('blogango_admin_entry_comments_manage', args=[entry.pk]))
        comments = response.context['comments']
        self.assertEqual(1, comments.count())


class TestStaticFiles(TestCase):
    """Check if app contains required static files"""
    def test_images(self):
        abs_path = finders.find('blogango/images/agiliq_blog_logo.png')
        self.assertTrue(staticfiles_storage.exists(abs_path))
        abs_path = finders.find('blogango/images/person_default.jpg')
        self.assertTrue(staticfiles_storage.exists(abs_path))

    def test_css(self):
        abs_path = finders.find('blogango/css/as_blog_styles.css')
        self.assertTrue(staticfiles_storage.exists(abs_path))
        abs_path = finders.find('blogango/css/prettify.css')


class TestFeedUrl(TestCase):
    """Check different feed urls and if they are being set in the page"""
    def setUp(self):
        self.blog = Blog(title="test", tag_line="new blog", entries_per_page=10,
                    recents=5, recent_comments=5)
        self.blog.save()
        self.c = Client()
        self.user = User.objects.create_user(username='gonecrazy', email='gonecrazy@gmail.com', password='gonecrazy')

    def test_blog_feed(self):
        response = self.c.get(reverse("blogango_feed"))
        self.assertEqual(response.status_code, 200)

    def test_feed_url_on_index(self):
        response = self.c.get(reverse("blogango_index"))
        self.assertGreater(response.content.find('/blog/rss/latest/'), 1)

def create_test_blog_entry(user):
    return BlogEntry.objects.create(**{'title': 'example post',
            'text': 'this is the test post',
            'publish_date': datetime.strptime("2011-09-22", "%Y-%m-%d"),
            'text_markup_type': "html",
            'created_by': user})


def create_test_comment(entry):
    Comment.objects.create(text="foo", comment_for=entry, is_public=True)



import urllib2, urllib
from django.utils import simplejson
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
import views


site = 'blogango.com'

def search (request):
    query_term = ""
    for term in request.GET['q']:
        query_term += term
    results = get_search_results('YLPjx2rV34F4hXcTnJYqYJUj9tANeqax76Ip2vADl9kKuByRNHgC4qafbATFoQ', query_term, site = site)
    print [result['Url'] for result in results]
    payload = {'results':results}
    return views.render('search.html', request, payload)

def get_search_results(appid, query, region ='us', type = 'all', results = 10, start = 0, format ='any', adult_ok = "", similar_ok = "", language = "", country = "", site = "", subscription = "", license = ''):
    base_url = u'http://search.yahooapis.com/WebSearchService/V1/webSearch?'
    params = locals()
    result = _query_yahoo(base_url, params)
    return result['ResultSet']['Result']

def _query_yahoo(base_url, params):
    params['output'] = 'json'
    payload = urllib.urlencode(params)
    url = base_url + payload
    print url
    response = urllib2.urlopen(url)
    result = simplejson.load(response)
    return result    
